@@ -1,18 +1,18 @@
 /* -*- c-file-style: "linux"; -*-
- * 
+ *
  * Copyright (C) 1998-2001 by Andrew Tridgell <tridge@samba.org>
  * Copyright (C) 2001-2002 by Martin Pool <mbp@samba.org>
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -27,12 +27,31 @@
 
 #include "rsync.h"
 
+extern int am_sender;
+extern int am_server;
+extern int am_daemon;
+extern int am_root;
 extern int module_id;
 extern int read_only;
 extern int verbose;
 extern int rsync_port;
-char *auth_user;
+extern int kludge_around_eof;
+extern int daemon_over_rsh;
+extern int list_only;
 extern int sanitize_paths;
+extern int filesfrom_fd;
+extern int remote_protocol;
+extern int protocol_version;
+extern int io_timeout;
+extern int orig_umask;
+extern int no_detach;
+extern int default_af_hint;
+extern char *bind_address;
+extern struct exclude_list_struct server_exclude_list;
+extern char *exclude_path_prefix;
+extern char *config_file;
+
+char *auth_user;
 
 /**
  * Run a client connected to an rsyncd.  The alternative to this
@@ -52,12 +71,10 @@ int start_socket_client(char *host, char *path, int argc, char *argv[])
 {
 	int fd, ret;
 	char *p, *user=NULL;
-	extern char *bind_address;
-	extern int default_af_hint;
-       
+
 	/* this is redundant with code in start_inband_exchange(), but
-	   this short-circuits a problem before we open a socket, and 
-	   the extra check won't hurt */
+	 * this short-circuits a problem before we open a socket, and
+	 * the extra check won't hurt */
 	if (*path == '/') {
 		rprintf(FERROR,"ERROR: The remote path must start with a module name not a /\n");
 		return -1;
@@ -70,18 +87,10 @@ int start_socket_client(char *host, char *path, int argc, char *argv[])
 		*p = 0;
 	}
 
-	if (verbose >= 2) {
-		/* FIXME: If we're going to use a socket program for
-		 * testing, then this message is wrong.  We need to
-		 * say something like "(except really using %s)" */
-		rprintf(FINFO, "opening tcp connection to %s port %d\n",
-			host, rsync_port);
-	}
-	fd = open_socket_out_wrapped (host, rsync_port, bind_address,
-				      default_af_hint);
-	if (fd == -1) {
+	fd = open_socket_out_wrapped(host, rsync_port, bind_address,
+				     default_af_hint);
+	if (fd == -1)
 		exit_cleanup(RERR_SOCKETIO);
-	}
 
 	ret = start_inband_exchange(user, path, fd, fd, argc);
 
@@ -95,11 +104,6 @@ int start_inband_exchange(char *user, char *path, int f_in, int f_out, int argc)
 	int sargc = 0;
 	char line[MAXPATHLEN];
 	char *p;
-	extern int remote_version;
-	extern int kludge_around_eof;
-	extern int am_sender;
-	extern int daemon_over_rsh;
-	extern int list_only;
 
 	if (argc == 0 && !am_sender)
 		list_only = 1;
@@ -112,32 +116,34 @@ int start_inband_exchange(char *user, char *path, int f_in, int f_out, int argc)
 	if (!user) user = getenv("USER");
 	if (!user) user = getenv("LOGNAME");
 
-	/* set daemon_over_rsh to false since we need to build the 
-	   true set of args passed through the rsh/ssh connection; 
-	   this is a no-op for direct-socket-connection mode */
+	/* set daemon_over_rsh to false since we need to build the
+	 * true set of args passed through the rsh/ssh connection;
+	 * this is a no-op for direct-socket-connection mode */
 	daemon_over_rsh = 0;
 	server_options(sargs, &sargc);
 
 	sargs[sargc++] = ".";
 
-	if (path && *path) 
+	if (path && *path)
 		sargs[sargc++] = path;
 
 	sargs[sargc] = NULL;
 
-	io_printf(f_out, "@RSYNCD: %d\n", PROTOCOL_VERSION);
+	io_printf(f_out, "@RSYNCD: %d\n", protocol_version);
 
 	if (!read_line(f_in, line, sizeof(line)-1)) {
 		rprintf(FERROR, "rsync: did not see server greeting\n");
 		return -1;
 	}
 
-	if (sscanf(line,"@RSYNCD: %d", &remote_version) != 1) {
+	if (sscanf(line,"@RSYNCD: %d", &remote_protocol) != 1) {
 		/* note that read_line strips of \n or \r */
 		rprintf(FERROR, "rsync: server sent \"%s\" rather than greeting\n",
 			line);
 		return -1;
 	}
+	if (protocol_version > remote_protocol)
+		protocol_version = remote_protocol;
 
 	p = strchr(path,'/');
 	if (p) *p = 0;
@@ -146,7 +152,7 @@ int start_inband_exchange(char *user, char *path, int f_in, int f_out, int argc)
 
 	/* Old servers may just drop the connection here,
 	 rather than sending a proper EXIT command.  Yuck. */
-	kludge_around_eof = list_only && (remote_version < 25);
+	kludge_around_eof = list_only && (protocol_version < 25);
 
 	while (1) {
 		if (!read_line(f_in, line, sizeof(line)-1)) {
@@ -185,8 +191,8 @@ int start_inband_exchange(char *user, char *path, int f_in, int f_out, int argc)
 	}
 	io_printf(f_out, "\n");
 
-	if (remote_version < 23) {
-		if (remote_version == 22 || (remote_version > 17 && !am_sender))
+	if (protocol_version < 23) {
+		if (protocol_version == 22 || !am_sender)
 			io_start_multiplex_in(f_in);
 	}
 
@@ -211,11 +217,6 @@ static int rsync_module(int f_in, int f_out, int i)
 	int start_glob=0;
 	int ret;
 	char *request=NULL;
-	extern int am_sender;
-	extern int am_server;
-	extern int am_daemon;
-	extern int remote_version;
-	extern int am_root;
 
 	if (!allow_access(addr, host, lp_hosts_allow(i), lp_hosts_deny(i))) {
 		rprintf(FERROR,"rsync denied on module %s from %s (%s)\n",
@@ -239,24 +240,25 @@ static int rsync_module(int f_in, int f_out, int i)
 		} else {
 			rprintf(FERROR,"max connections (%d) reached\n",
 				lp_max_connections(i));
-			io_printf(f_out, "@ERROR: max connections (%d) reached - try again later\n", lp_max_connections(i));
+			io_printf(f_out, "@ERROR: max connections (%d) reached - try again later\n",
+				lp_max_connections(i));
 		}
 		return -1;
 	}
 
-	
+
 	auth_user = auth_server(f_in, f_out, i, addr, "@RSYNCD: AUTHREQD ");
 
 	if (!auth_user) {
 		rprintf(FERROR,"auth failed on module %s from %s (%s)\n",
 			name, host, addr);
 		io_printf(f_out, "@ERROR: auth failed on module %s\n", name);
-		return -1;		
+		return -1;
 	}
 
 	module_id = i;
 
-	am_root = (getuid() == 0);
+	am_root = (MY_UID() == 0);
 
 	if (am_root) {
 		p = lp_uid(i);
@@ -265,7 +267,7 @@ static int rsync_module(int f_in, int f_out, int i)
 				rprintf(FERROR,"Invalid uid %s\n", p);
 				io_printf(f_out, "@ERROR: invalid uid %s\n", p);
 				return -1;
-			} 
+			}
 			uid = atoi(p);
 		}
 
@@ -275,29 +277,38 @@ static int rsync_module(int f_in, int f_out, int i)
 				rprintf(FERROR,"Invalid gid %s\n", p);
 				io_printf(f_out, "@ERROR: invalid gid %s\n", p);
 				return -1;
-			} 
+			}
 			gid = atoi(p);
 		}
 	}
-        
-        /* TODO: If we're not root, but the configuration requests
-         * that we change to some uid other than the current one, then
-         * log a warning. */
 
-        /* TODO: Perhaps take a list of gids, and make them into the
-         * supplementary groups. */
+	/* TODO: If we're not root, but the configuration requests
+	 * that we change to some uid other than the current one, then
+	 * log a warning. */
+
+	/* TODO: Perhaps take a list of gids, and make them into the
+	 * supplementary groups. */
+
+	exclude_path_prefix = use_chroot? "" : lp_path(i);
+	if (*exclude_path_prefix == '/' && !exclude_path_prefix[1])
+		exclude_path_prefix = "";
 
 	p = lp_include_from(i);
-	add_exclude_file(p, 1, 1);
+	add_exclude_file(&server_exclude_list, p,
+			 XFLG_FATAL_ERRORS | XFLG_DEF_INCLUDE);
 
 	p = lp_include(i);
-	add_include_line(p);
+	add_exclude(&server_exclude_list, p,
+		    XFLG_WORD_SPLIT | XFLG_DEF_INCLUDE);
 
 	p = lp_exclude_from(i);
-	add_exclude_file(p, 1, 0);
+	add_exclude_file(&server_exclude_list, p,
+			 XFLG_FATAL_ERRORS);
 
 	p = lp_exclude(i);
-	add_exclude_line(p);
+	add_exclude(&server_exclude_list, p, XFLG_WORD_SPLIT);
+
+	exclude_path_prefix = NULL;
 
 	log_init();
 
@@ -320,14 +331,14 @@ static int rsync_module(int f_in, int f_out, int i)
 			return -1;
 		}
 
-		if (!push_dir("/", 0)) {
-                        rsyserr(FERROR, errno, "chdir %s failed\n", lp_path(i));
+		if (!push_dir("/")) {
+			rsyserr(FERROR, errno, "chdir %s failed\n", lp_path(i));
 			io_printf(f_out, "@ERROR: chdir failed\n");
 			return -1;
 		}
 
 	} else {
-		if (!push_dir(lp_path(i), 0)) {
+		if (!push_dir(lp_path(i))) {
 			rsyserr(FERROR, errno, "chdir %s failed\n", lp_path(i));
 			io_printf(f_out, "@ERROR: chdir failed\n");
 			return -1;
@@ -336,16 +347,6 @@ static int rsync_module(int f_in, int f_out, int i)
 	}
 
 	if (am_root) {
-#ifdef HAVE_SETGROUPS
-		/* Get rid of any supplementary groups this process
-		 * might have inheristed. */
-		if (setgroups(0, NULL)) {
-			rsyserr(FERROR, errno, "setgroups failed");
-			io_printf(f_out, "@ERROR: setgroups failed\n");
-			return -1;
-		}
-#endif
-
 		/* XXXX: You could argue that if the daemon is started
 		 * by a non-root user and they explicitly specify a
 		 * gid, then we should try to change to that gid --
@@ -361,6 +362,15 @@ static int rsync_module(int f_in, int f_out, int i)
 			io_printf(f_out, "@ERROR: setgid failed\n");
 			return -1;
 		}
+#ifdef HAVE_SETGROUPS
+		/* Get rid of any supplementary groups this process
+		 * might have inheristed. */
+		if (setgroups(1, &gid)) {
+			rsyserr(FERROR, errno, "setgroups failed");
+			io_printf(f_out, "@ERROR: setgroups failed\n");
+			return -1;
+		}
+#endif
 
 		if (setuid(uid)) {
 			rsyserr(FERROR, errno, "setuid %d failed", (int) uid);
@@ -368,7 +378,7 @@ static int rsync_module(int f_in, int f_out, int i)
 			return -1;
 		}
 
-		am_root = (getuid() == 0);
+		am_root = (MY_UID() == 0);
 	}
 
 	io_printf(f_out, "@RSYNCD: OK\n");
@@ -408,22 +418,12 @@ static int rsync_module(int f_in, int f_out, int i)
 		}
 	}
 
-	if (sanitize_paths) {
-		/*
-		 * Note that this is applied to all parameters, whether or not
-		 *    they are filenames, but no other legal parameters contain
-		 *    the forms that need to be sanitized so it doesn't hurt;
-		 *    it is not known at this point which parameters are files
-		 *    and which aren't.
-		 */
-		for (i = 1; i < argc; i++) {
-			sanitize_path(argv[i], NULL);
-		}
-	}
-
-        argp = argv;
+	argp = argv;
 	ret = parse_arguments(&argc, (const char ***) &argp, 0);
 
+	if (filesfrom_fd == 0)
+		filesfrom_fd = f_in;
+
 	if (request) {
 		if (*auth_user) {
 			rprintf(FINFO,"rsync %s %s from %s@%s (%s)\n",
@@ -439,29 +439,29 @@ static int rsync_module(int f_in, int f_out, int i)
 
 #ifndef DEBUG
 	/* don't allow the logs to be flooded too fast */
-	if (verbose > 1) verbose = 1;
+	if (verbose > lp_max_verbosity())
+		verbose = lp_max_verbosity();
 #endif
 
-	if (remote_version < 23) {
-		if (remote_version == 22 || (remote_version > 17 && am_sender))
+	if (protocol_version < 23) {
+		if (protocol_version == 22 || am_sender)
 			io_start_multiplex_out(f_out);
 	}
-        
-        /* For later protocol versions, we don't start multiplexing
-         * until we've configured nonblocking in start_server.  That
-         * means we're in a sticky situation now: there's no way to
-         * convey errors to the client. */
 
-        /* FIXME: Hold off on reporting option processing errors until
-         * we've set up nonblocking and multiplexed IO and can get the
-         * message back to them. */
+	/* For later protocol versions, we don't start multiplexing
+	 * until we've configured nonblocking in start_server.  That
+	 * means we're in a sticky situation now: there's no way to
+	 * convey errors to the client. */
+
+	/* FIXME: Hold off on reporting option processing errors until
+	 * we've set up nonblocking and multiplexed IO and can get the
+	 * message back to them. */
 	if (!ret) {
-                option_error();
-                exit_cleanup(RERR_UNSUPPORTED);
+		option_error();
+		exit_cleanup(RERR_UNSUPPORTED);
 	}
 
 	if (lp_timeout(i)) {
-		extern int io_timeout;
 		io_timeout = lp_timeout(i);
 	}
 
@@ -476,13 +476,12 @@ static void send_listing(int fd)
 {
 	int n = lp_numservices();
 	int i;
-	extern int remote_version;
 
 	for (i=0;i<n;i++)
 		if (lp_list(i))
-		    io_printf(fd, "%-15s\t%s\n", lp_name(i), lp_comment(i));
+			io_printf(fd, "%-15s\t%s\n", lp_name(i), lp_comment(i));
 
-	if (remote_version >= 25)
+	if (protocol_version >= 25)
 		io_printf(fd,"@RSYNCD: EXIT\n");
 }
 
@@ -494,9 +493,6 @@ int start_daemon(int f_in, int f_out)
 	char line[200];
 	char *motd;
 	int i = -1;
-	extern char *config_file;
-	extern int remote_version;
-	extern int am_server;
 
 	if (!lp_load(config_file, 0)) {
 		exit_cleanup(RERR_SYNTAX);
@@ -510,7 +506,7 @@ int start_daemon(int f_in, int f_out)
 		set_nonblocking(f_in);
 	}
 
-	io_printf(f_out, "@RSYNCD: %d\n", PROTOCOL_VERSION);
+	io_printf(f_out, "@RSYNCD: %d\n", protocol_version);
 
 	motd = lp_motd_file();
 	if (motd && *motd) {
@@ -530,10 +526,12 @@ int start_daemon(int f_in, int f_out)
 		return -1;
 	}
 
-	if (sscanf(line,"@RSYNCD: %d", &remote_version) != 1) {
+	if (sscanf(line,"@RSYNCD: %d", &remote_protocol) != 1) {
 		io_printf(f_out, "@ERROR: protocol startup error\n");
 		return -1;
-	}	
+	}
+	if (protocol_version > remote_protocol)
+		protocol_version = remote_protocol;
 
 	while (i == -1) {
 		line[0] = 0;
@@ -544,7 +542,7 @@ int start_daemon(int f_in, int f_out)
 		if (!*line || strcmp(line,"#list")==0) {
 			send_listing(f_out);
 			return -1;
-		} 
+		}
 
 		if (*line == '#') {
 			/* it's some sort of command that I don't understand */
@@ -565,19 +563,16 @@ int start_daemon(int f_in, int f_out)
 
 int daemon_main(void)
 {
-	extern char *config_file;
-	extern int orig_umask;
 	char *pid_file;
-	extern int no_detach;
 
 	if (is_a_socket(STDIN_FILENO)) {
 		int i;
 
 		/* we are running via inetd - close off stdout and
-		   stderr so that library functions (and getopt) don't
-		   try to use them. Redirect them to /dev/null */
+		 * stderr so that library functions (and getopt) don't
+		 * try to use them. Redirect them to /dev/null */
 		for (i=1;i<3;i++) {
-			close(i); 
+			close(i);
 			open("/dev/null", O_RDWR);
 		}
 
@@ -585,7 +580,7 @@ int daemon_main(void)
 	}
 
 	if (!no_detach)
-	    become_daemon();
+		become_daemon();
 
 	if (!lp_load(config_file, 1)) {
 		exit_cleanup(RERR_SYNTAX);
@@ -594,24 +589,23 @@ int daemon_main(void)
 	log_init();
 
 	rprintf(FINFO, "rsyncd version %s starting, listening on port %d\n",
-		RSYNC_VERSION,
-                rsync_port);
-        /* TODO: If listening on a particular address, then show that
-         * address too.  In fact, why not just do inet_ntop on the
-         * local address??? */
+		RSYNC_VERSION, rsync_port);
+	/* TODO: If listening on a particular address, then show that
+	 * address too.  In fact, why not just do inet_ntop on the
+	 * local address??? */
 
 	if (((pid_file = lp_pid_file()) != NULL) && (*pid_file != '\0')) {
 		char pidbuf[16];
 		int fd;
-		int pid = (int) getpid();
+		pid_t pid = getpid();
 		cleanup_set_pid(pid);
 		if ((fd = do_open(lp_pid_file(), O_WRONLY|O_CREAT|O_TRUNC,
 					0666 & ~orig_umask)) == -1) {
-		    cleanup_set_pid(0);
-		    rsyserr(FLOG, errno, "failed to create pid file %s", pid_file);
-		    exit_cleanup(RERR_FILEIO);
+			cleanup_set_pid(0);
+			rsyserr(FLOG, errno, "failed to create pid file %s", pid_file);
+			exit_cleanup(RERR_FILEIO);
 		}
-		snprintf(pidbuf, sizeof(pidbuf), "%d\n", pid);
+		snprintf(pidbuf, sizeof pidbuf, "%ld\n", (long)pid);
 		write(fd, pidbuf, strlen(pidbuf));
 		close(fd);
 	}
@@ -619,4 +613,3 @@ int daemon_main(void)
 	start_accept_loop(rsync_port, start_daemon);
 	return -1;
 }
-
